@@ -114,7 +114,7 @@ collector_runner(void *s)
 		server_send(submission, strdup(metric), 1);
 
 	nextcycle = time(NULL) + collector_interval;
-	while (keep_running) {
+	while (__sync_bool_compare_and_swap(&keep_running, 1, 1)) {
 		if (cluster_refresh_pending) {
 			char *stub = router_getcollectorstub(pending_router);
 			server **newservers = router_getservers(pending_router);
@@ -225,6 +225,8 @@ collector_runner(void *s)
 					snprintf(destbuf, sizeof(destbuf), "%s:%u-udp",
 							server_ip(srvs[i]), server_port(srvs[i]));
 					break;
+				default:
+					assert(0);  /* help code analysis tools */
 			}
 			for (p = destbuf; *p != '\0'; p++)
 				if (*p == '.')
@@ -359,6 +361,8 @@ collector_writer(void *unused)
 			}
 			totconn = dispatch_get_accepted_connections();
 			totdisc = dispatch_get_closed_connections();
+			if (dticks == 0)
+				dticks = 1;  /* for Coverity */
 			printf("%5zu %7zu  "   /* metrics in */
 					"%5zu %7zu  "  /* metrics out */
 					"%5zu %7zu  "  /* metrics dropped */
@@ -458,6 +462,6 @@ collector_start(dispatcher **d, router *rtr, server *submission, char cum)
 void
 collector_stop(void)
 {
-	keep_running = 0;
+	__sync_bool_compare_and_swap(&keep_running, 1, 0);
 	pthread_join(collectorid, NULL);
 }
