@@ -123,32 +123,33 @@ dispatch_addlistener(int sock)
 	connection *newconn;
 	int c;
 
-	newconn = malloc(sizeof(connection));
-	if (newconn == NULL)
-		return 1;
-	(void) fcntl(sock, F_SETFL, O_NONBLOCK);
-	newconn->sock = sock;
-	newconn->takenby = 0;
-	newconn->buflen = 0;
-
 	pthread_rwlock_wrlock(&listenerslock);
 	if (listeners == NULL) {
-		listeners = malloc(sizeof(connection *) * MAX_LISTENERS);
+		if ((listeners = malloc(sizeof(connection *) * MAX_LISTENERS)) == NULL)
+		{
+			pthread_rwlock_unlock(&listenerslock);
+			return 1;
+		}
 		for (c = 0; c < MAX_LISTENERS; c++)
 			listeners[c] = NULL;
 	}
-	if (listeners == NULL) {
-		pthread_rwlock_unlock(&listenerslock);
-		return 1;
-	}
 	for (c = 0; c < MAX_LISTENERS; c++) {
 		if (listeners[c] == NULL) {
+			newconn = malloc(sizeof(connection));
+			if (newconn == NULL) {
+				pthread_rwlock_unlock(&listenerslock);
+				return 1;
+			}
+			(void) fcntl(sock, F_SETFL, O_NONBLOCK);
+			newconn->sock = sock;
+			newconn->takenby = 0;
+			newconn->buflen = 0;
+
 			listeners[c] = newconn;
 			break;
 		}
 	}
 	if (c == MAX_LISTENERS) {
-		free(newconn);
 		logerr("cannot add new listener: "
 				"no more free listener slots (max = %zu)\n",
 				MAX_LISTENERS);
@@ -333,6 +334,10 @@ dispatch_process_dests(connection *conn, dispatcher *self, struct timeval now)
 		/* force after timeout */
 		force = timediff(conn->lastwork, now) > conn->maxsenddelay;
 		for (i = 0; i < conn->destlen; i++) {
+			tracef("dispatcher %d, connfd %d, metric %s, queueing to %s:%d\n",
+					self->id, conn->sock, conn->dests[i].metric,
+					server_ip(conn->dests[i].dest),
+					server_port(conn->dests[i].dest));
 			if (server_send(conn->dests[i].dest, conn->dests[i].metric, force) == 0)
 				break;
 		}
@@ -403,8 +408,11 @@ dispatch_connection(connection *conn, dispatcher *self, struct timeval start)
 						(sizeof(conn->buf) - 1) - conn->buflen)) > 0
 	   )
 	{
-		if (len > 0)
+		if (len > 0) {
 			conn->buflen += len;
+			tracef("dispatcher %d, connfd %d, read %d bytes from socket\n",
+					self->id, conn->sock, len);
+		}
 
 		/* metrics look like this: metric_path value timestamp\n
 		 * due to various messups we need to sanitise the
@@ -431,11 +439,15 @@ dispatch_connection(connection *conn, dispatcher *self, struct timeval start)
 				*q = '\0';  /* can do this because we substract one from buf */
 
 				/* perform routing of this metric */
+				tracef("dispatcher %d, connfd %d, metric %s",
+						self->id, conn->sock, conn->metric);
 				__sync_add_and_fetch(&(self->blackholes),
 						router_route(self->rtr,
 						conn->dests, &conn->destlen, CONN_DESTS_SIZE,
 						conn->srcaddr,
 						conn->metric, firstspace));
+				tracef("dispatcher %d, connfd %d, destinations %zd\n",
+						self->id, conn->sock, conn->destlen);
 
 				/* restart building new one from the start */
 				q = conn->metric;
@@ -671,6 +683,12 @@ dispatch_new(char id, enum conntype type, router *r, char *allowed_chars)
 	return ret;
 }
 
+void
+dispatch_set_bufsize(unsigned int nsockbufsize)
+{
+	sockbufsize = nsockbufsize;
+}
+
 static char globalid = 0;
 
 /**
@@ -678,10 +696,9 @@ static char globalid = 0;
  * (and putting them on the queue for handling the connections).
  */
 dispatcher *
-dispatch_new_listener(unsigned int nsockbufsize)
+dispatch_new_listener(void)
 {
 	char id = globalid++;
-	sockbufsize = nsockbufsize;
 	return dispatch_new(id, LISTENER, NULL, NULL);
 }
 
