@@ -76,6 +76,10 @@ relaylog(enum logdst dest, const char *fmt, ...)
 	char console = 0;
 	int ret;
 
+	/* briefly stall if we're swapping fds */
+	while (!__sync_add_and_fetch(&relay_can_log, 0))
+		usleep((100 + (rand() % 200)) * 1000);  /* 100ms - 300ms */
+
 	switch (dest) {
 		case LOGOUT:
 			dst = relay_stdout;
@@ -90,10 +94,6 @@ relaylog(enum logdst dest, const char *fmt, ...)
 	}
 	assert(dst != NULL);
 
-	/* briefly stall if we're swapping fds */
-	while (!__sync_add_and_fetch(&relay_can_log, 0))
-		usleep((100 + (rand() % 200)) * 1000);  /* 100ms - 300ms */
-
 	time(&now);
 	localtime_r(&now, &tm_now);
 	len = strftime(prefix, sizeof(prefix), "[%Y-%m-%d %H:%M:%S]", &tm_now);
@@ -357,7 +357,14 @@ main(int argc, char * const argv[])
 				do_version();
 				break;
 			case 'd':
-				mode |= MODE_DEBUG;
+				/* secret support for -dd (just trace) and -ddd (debug
+				 * and trace) */
+				if (mode & MODE_DEBUG) {
+					mode |= MODE_TRACE;
+					mode &= ~MODE_DEBUG;
+				} else {
+					mode |= MODE_DEBUG;
+				}
 				break;
 			case 'm':
 				smode = SUB;
@@ -661,6 +668,10 @@ main(int argc, char * const argv[])
 				allowed_chars);
 	if (mode & MODE_DEBUG)
 		fprintf(relay_stdout, "    debug = true\n");
+#ifdef ENABLE_TRACE
+	if (mode & MODE_TRACE)
+		fprintf(relay_stdout, "    trace = true\n");
+#endif
 	else if (mode & MODE_SUBMISSION)
 		fprintf(relay_stdout, "    submission = true\n");
 	else if (mode & MODE_DAEMON)
@@ -679,16 +690,16 @@ main(int argc, char * const argv[])
 
 	aggrs = router_getaggregators(rtr);
 	numaggregators = aggregator_numaggregators(aggrs);
-	if (numaggregators > 10 && mode & MODE_DEBUG) {
+	if (numaggregators > 10 && !(mode & MODE_DEBUG)) {
 		fprintf(relay_stdout, "parsed configuration follows:\n"
 				"(%zu aggregations with %zu computations omitted "
 				"for brevity)\n",
 				numaggregators, aggregator_numcomputes(aggrs));
-		router_printconfig(rtr, relay_stdout, 0);
+		router_printconfig(rtr, relay_stdout, PMODE_NORM);
 	} else {
 		fprintf(relay_stdout, "parsed configuration follows:\n");
 		router_printconfig(rtr, relay_stdout,
-				1 + (mode & MODE_DEBUG ? 2 : 0));
+				PMODE_AGGR + (mode & MODE_DEBUG ? PMODE_DEBUG : PMODE_NORM));
 	}
 	fprintf(relay_stdout, "\n");
 
@@ -742,6 +753,7 @@ main(int argc, char * const argv[])
 				listenport, strerror(errno));
 		return -1;
 	}
+	dispatch_set_bufsize(sockbufsize);
 	for (ch = 0; ch < stream_socklen; ch++) {
 		if (dispatch_addlistener(stream_sock[ch]) != 0) {
 			logerr("failed to add listener\n");
@@ -754,7 +766,7 @@ main(int argc, char * const argv[])
 			return -1;
 		}
 	}
-	if ((workers[0] = dispatch_new_listener(sockbufsize)) == NULL)
+	if ((workers[0] = dispatch_new_listener()) == NULL)
 		logerr("failed to add listener\n");
 
 	if (allowed_chars == NULL)
