@@ -162,6 +162,7 @@ ra_malloc(router *rtr, size_t sz)
 	size_t nsz;
 
 #define ra_alloc(RA, SZ) { \
+		assert(SZ >= 0); \
 		nsz = 256 * 1024; \
 		if (SZ > nsz) \
 			nsz = ((SZ / 1024) + 1) * 1024; \
@@ -411,6 +412,7 @@ router_readconfig(router *orig,
 		ret->routes = NULL;
 		ret->aggregators = NULL;
 		ret->srvrs = NULL;
+		ret->clusters = NULL;
 
 		/* create virtual blackhole cluster */
 		cl = ra_malloc(ret, sizeof(cluster));
@@ -1972,8 +1974,8 @@ router_optimise(router *r)
 	blocks->next = NULL;
 	rlast = NULL;
 	for (rwalk = r->routes; rwalk != NULL; rwalk = rnext) {
-		/* matchall rules cannot be in a group */
-		if (rwalk->matchtype == MATCHALL) {
+		/* matchall and rewrite rules cannot be in a group (issue #218) */
+		if (rwalk->matchtype == MATCHALL || rwalk->dests->cl->type == REWRITE) {
 			blast->next = malloc(sizeof(block));
 			blast->next->prev = blast;
 			blast = blast->next;
@@ -2041,9 +2043,9 @@ router_optimise(router *r)
 		while (
 				p >= rwalk->pattern && b - pblock < sizeof(pblock) &&
 				(
-				(*p >= 'a' && *p <= 'z') ||
-				(*p >= 'A' && *p <= 'Z') ||
-				*p == '_'
+					(*p >= 'a' && *p <= 'z') ||
+					(*p >= 'A' && *p <= 'Z') ||
+					*p == '_'
 				)
 			  )
 		{
@@ -2290,7 +2292,7 @@ router_printconfig(router *rtr, FILE *f, char pmode)
 						PPROTO);
 		}
 		fprintf(f, "    ;\n");
-		if (pmode & 2) {
+		if (pmode & PMODE_HASH) {
 			if (c->type == CARBON_CH ||
 					c->type == FNV1A_CH ||
 					c->type == JUMP_CH)
@@ -2308,10 +2310,10 @@ router_printconfig(router *rtr, FILE *f, char pmode)
 			char stubname[48];
 			char percentile[16];
 
-			if (!(pmode & 1))
+			if (!(pmode & PMODE_AGGR))
 				continue;
 
-			if (pmode & 2 || r->dests->next == NULL) {
+			if (pmode & PMODE_STUB || r->dests->next == NULL) {
 				stubname[0] = '\0';
 			} else {
 				snprintf(stubname, sizeof(stubname),
@@ -2352,7 +2354,7 @@ router_printconfig(router *rtr, FILE *f, char pmode)
 						"<unknown>",
 						ac->metric + strlen(stubname));
 			}
-			if (!(pmode & 2) && r->dests->next != NULL) {
+			if (!(pmode & PMODE_STUB) && r->dests->next != NULL) {
 				destinations *dn = r->dests->next;
 				fprintf(f, "    send to");
 				if (dn->next == NULL) {
@@ -2383,10 +2385,17 @@ router_printconfig(router *rtr, FILE *f, char pmode)
 			fprintf(f, "# common pattern group '%s' "
 					"contains %zu aggregations/matches\n",
 					++b, cnt);
+			if (mode & PMODE_AGGR) {
+				router srtr;
+				memset(&srtr, 0, sizeof(srtr));
+				srtr.routes = r->dests->cl->members.routes;
+				/* recurse */
+				router_printconfig(&srtr, f, pmode);
+			}
 		} else if (r->dests->cl->type == AGGRSTUB ||
 				r->dests->cl->type == STATSTUB)
 		{
-			if (pmode & 2) {
+			if (pmode & PMODE_STUB) {
 				fprintf(f, "# stub match for aggregate/statistics rule "
 						"with send to\n");
 				fprintf(f, "match ^%s\n    send to", r->pattern);
@@ -2454,6 +2463,7 @@ router_printdiffs(router *old, router *new, FILE *out)
 	char buf[1024];
 	size_t len;
 	int ret;
+	mode_t mask;
 
 	/* First idea was to printconfig both old and new, and run diff -u
 	 * on them.  Simple and effective.  Does require diff and two files
@@ -2463,7 +2473,9 @@ router_printdiffs(router *old, router *new, FILE *out)
 		tmp = "/tmp";
 
 	snprintf(patho, sizeof(patho), "%s/carbon-c-relay_route.XXXXXX", tmp);
+	mask = umask(S_IWGRP | S_IWOTH);
 	fd = mkstemp(patho);
+	umask(mask);
 	if (fd < 0) {
 		logerr("failed to create temporary file: %s\n", strerror(errno));
 		return 1;
@@ -2474,11 +2486,13 @@ router_printdiffs(router *old, router *new, FILE *out)
 				patho, strerror(errno));
 		return 1;
 	}
-	router_printconfig(old, f, 1);
+	router_printconfig(old, f, PMODE_AGGR);
 	fclose(f);
 
 	snprintf(pathn, sizeof(pathn), "%s/carbon-c-relay_route.XXXXXX", tmp);
+	mask = umask(S_IWGRP | S_IWOTH);
 	fd = mkstemp(pathn);
+	umask(mask);
 	if (fd < 0) {
 		logerr("failed to create temporary file: %s\n", strerror(errno));
 		return 1;
@@ -2489,14 +2503,16 @@ router_printdiffs(router *old, router *new, FILE *out)
 				pathn, strerror(errno));
 		return 1;
 	}
-	router_printconfig(new, f, 1);
+	router_printconfig(new, f, PMODE_AGGR);
 	fclose(f);
 
 	/* diff and print its output */
 	snprintf(buf, sizeof(buf), "diff -u %s %s", patho, pathn);
 	f = popen(buf, "r");
 	while ((len = fread(buf, 1, sizeof(buf) - 1, f)) > 0) {
-		fwrite(buf, len, 1, out);
+		if (fwrite(buf, len, 1, out) == 0)
+			/* ignore */
+			;
 	}
 	ret = pclose(f);
 
